<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Services\QuoteSheetGrid\ExcludeItemsWithZeroPriceService as Service;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ExcludeItemsWithZeroPriceController {
  private $service;
  private $util;

  public function __construct(Service $service, Util $util) {
    $this->service = $service;
    $this->util    = $util;
  }

  public function excludeItemsWithZeroPrice(Request $request, Response $response) {
    $data = $request->getParameters();

    try {
      $conditions = [
        "CDFILIAL"      => $data["CDFILIAL"],
        "NRORG"         => $data["NRORG"],
        "NRCOLEPRECO"   => $data["NRCOLEPRECO"],
        "CDFILIAL"      => $data["CDFILIAL"],
        "NRCOLEPRECO"   => $data["NRCOLEPRECO"],
        "NRORG"         => $data["NRORG"]
      ];

      $zeroPrice = $this->util->filteredDataset($conditions, "verifyZeroPrice")[0];

      if ($zeroPrice["CTTOTITM"] === $zeroPrice["CTTOTZERO"] ) {
        return $response->setError(new Error("sup::OPERATION_NOT_ALLOWED", 401));
      }

      $this->service->deleteItemPriceZero($data);

      return $response->addDataSet(new DataSet("excludeItemsWithZeroPrice", ["message" => "Itens com preço zerado excluidos com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao deletar itens com preço zerado: " . $exception->getMessage(), 500);
    }
  }
}