<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Services\QuoteSheetGrid\PriceAdjustmentService as Service;
use SupNext\Controllers\QuoteSheetGrid\UpdateProductController as UpdateProduct;
use SupNext\Controllers\QuoteSheetGrid\AddRegionController as AddRegion;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class PriceAdjustmentController {
  private $service;
  private $updateProduct;
  private $addRegion;
  private $util;

  public function __construct(Service $service, UpdateProduct $updateProduct, AddRegion $addRegion, Util $util) {
    $this->service       = $service;
    $this->updateProduct = $updateProduct;
    $this->addRegion     = $addRegion;
    $this->util          = $util;
  }

  public function adjustPrice(Request $request, Response $response) {
    $data = $request->getParameters()["params"];
    $data["DSRDMUSUMCOT"] = null;
    $data["DSENDUSUMCOT"] = null;
    $data["DSNMUSUMCOT"]  = null;
    $data["DSMETUSUMCOT"] = null;

    try {
      if ($data["REAJUSTE"] != 0) {
        $this->service->insertItemCotacaoAdjustment($data);
        $this->service->updatePriceDate($data);

        $conditions = [
          "CDPRODINI"   => $data["CDPRODINI"],
          "CDPRODFIN"   => $data["CDPRODFIN"],
          "NRCOLEPRECO" => $data["NRCOLEPRECO"],
          "CDFORNECED"  => $data["CDFORNECED"],
          "CDREGIAO"    => $data["CDREGIAO"],
          "NRORG"       => $data["NRORG"]
        ];

        $productsSupplyTek = $this->util->filteredDataSet($conditions, "verifyProdOnSupplyTek");
        if (sizeof($productsSupplyTek) > 0) {
          $this->updateProduct->saveNotificationOnSupplyTek($data, $productsSupplyTek, "multiple");
        }

        $conditions = [
          "CDREGIAO"    => $data["CDREGIAO"],
          "CDFILIAL"    => $data["CDFILIAL"],
          "NRCOLEPRECO" => $data["NRCOLEPRECO"],
          "CDPRODINI"   => $data["CDPRODINI"],
          "CDPRODFIN"   => $data["CDPRODFIN"],
          "CDFORNECED"  => $data["CDFORNECED"],
          "NRORG"       => $data["NRORG"]
        ];

        $taxes = $this->util->filteredDataSet($conditions, "getTaxQuote");

        foreach ($taxes as $tax) {
          $conditions = [
            "CDFILIAL"  => $data["CDFILIAL"],
            "CDPRODUTO" => $data["CDPRODUTO"],
            "NRORG"     => $data["NRORG"]
          ];

          $issuerAF = $this->util->filteredDataSet($conditions, "getIssuerAF");

          if (empty($issuerAF)) {
            $conditions = [
              "CDFILIAL" => $data["CDFILIAL"],
              "NRORG"    => $data["NRORG"]
            ];

            $issuerAF = $this->util->filteredDataSet($conditions, "getIssuerAFDefault");
          }

          $conditions = [
            "CDEMPRESA" => $issuerAF[0]["CDEMPRESA"],
            "NRORG"     => $data["NRORG"]
          ];

          $companyInfo = $this->util->filteredDataSet($conditions, "getCompanyInfo");

          $paramsProc = [
            "CDEMPRESA"   => $companyInfo ? $companyInfo[0]["CDEMPRESA"]  : null,
            "CDINSCESTA"  => $companyInfo ? $companyInfo[0]["CDINSCESTA"] : null,
            "CDFILCENT"   => $data["CDFILCENT"],
            "NRCOLEPRECO" => $data["NRCOLEPRECO"],
            "CDFORNECED"  => $data["CDFORNECED"],
            "CDREGIAO"    => $data["CDREGIAO"],
            "CDPRODUTO"   => $tax["CDPRODUTO"],
            "CDMARCA"     => $tax["CDMARCA"],
            "DTCOLEPREC"  => $data["DTCOLEPREC"]
          ];

          $this->addRegion->executeProcedureItemcol($paramsProc);
        }
      }

      return $response->addDataSet(new DataSet("priceAdjustment", ["message" => "Ajuste dos preços realizado com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao ajustar preço: " . $exception->getMessage(), 500);
    }
  }
}
