<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use SupNext\Controllers\QR2StrategySAAS;
use SupNext\Util\Util;

use Zeedhi\Framework\Controller\Exception;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Controller\Report;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\Report\Strategy\BirtStrategy;

class RemoteReportController extends Report
{
    protected $reportService;
    private QR2StrategySAAS $qr2Strategy;
    private BirtStrategy $birtStrategy;
    private Util $util;

    public function __construct(
        ReportService $reportService,
        QR2StrategySAAS $qr2Strategy,
        BirtStrategy $birtStrategy,
        Util $util
    ) {
        parent::__construct($reportService);
        $this->reportService = $reportService;
        $this->qr2Strategy   = $qr2Strategy;
        $this->birtStrategy  = $birtStrategy;
        $this->util          = $util;
    }

    /**
     * @throws Exception
     * @throws \Exception
     */
    public function beforeProcessReport(Request\Row $request, Response $response): void
    {
        $data = $request->getRow();

        $this->qr2Strategy->setQr2Type($data["TYPE"]);
        $this->birtStrategy->setReportFormat($data["TYPE"]);
        $stNomeRel = $this->getReportName($data);

        if ($data["OPERADOR"] != "T") {
            $nrlistaForn = $this->util->createListValues($data["OPERADOR"]);
            $data["OPERADOR"] = "_NRLISTA=" . $nrlistaForn;
        }

        $this->reportMapping[$stNomeRel] =
            [
                "strategy"         => "BIRT",
                "parameterMapping" => [
                    "OPERADOR"   => "L_CDOPERADOR",
                    "P_DATAINIC" => "P_DATAINIC",
                    "P_DATAFINA" => "P_DATAFINA",
                    "DTINI"      => "P_DTI",
                    "DTFIM"      => "P_DTF",
                    "NR_COTACAO" => "P_NRCOLEPRLOG",
                    "NRORG"      => "P_NRORG"
                ]
            ];
    }
}
