<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use Exception;

use SupNext\Services\QuoteSheetGrid\ReplicateQuotationService as Service;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ReplicateQuotationController
{
    private Service $service;
    private Util $util;

    public function __construct(Service $service, Util $util)
    {
        $this->service = $service;
        $this->util = $util;
    }

    /**
     * @throws Exception
     */
    public function replicateQuotation(Request $request, Response $response): void
    {
        $data = $request->getParameters();

        try {
            $data["REAJUSTE"] = (float)$data["REAJUSTE"];
            $data["NRSEMAINIC"] = (string)$data["NRSEMAINIC"];
            $data["NRSEMAFINAL"] = (string)$data["NRSEMAFINAL"];
            $data = $this->util->implodeFields($data, ["CDREGIAO", "CDREGIAOOL"]);

            if (isset($data["DUPLICAOL"]) && $data["DUPLICAOL"] === "S") $this->checkOLRegion($data, $response);

            $dupQuotation = $this->service->procReplicateQuotation($data);

            $response->addDataSet(new DataSet("replicateQuotation", ["message" => "Operação realizada com sucesso.", $dupQuotation, "status_code" => 200]));
        } catch (Exception $exception) {
            throw new Exception("Erro ao replicar cotação: " . $exception->getMessage(), 500);
        }
    }

    public function checkOLRegion($data, $response): void
    {
        $OLSupplier = isset($data["CDFORNECED"]) ? ($data["CDFORNECED"]) : NULL;
        $conditions = [
            "P_CDFORNOPELOG" => $OLSupplier
        ];

        $OLRegion = $this->util->filteredDataSet($conditions, "OLRegionField");

        if (!count($OLRegion)) {
            $response->setError(new Error("sup::NO_REGION_REGISTERED_FOR_LOGISTICS_OPERATOR", 400));
        }
    }
}