<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use Teknisa\Libs\Util\Environment;

use SupNext\Util\Util;
use SupNext\Services\QuoteSheetGrid\UpdateProductService as Service;
use SupNext\Controllers\QuoteSheetGrid\AddProductController as AddProduct;
use SupNext\Controllers\QuoteSheetGrid\AddRegionController as AddRegion;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class UpdateProductController {
  private $environment;
  private $util;
  private $service;
  private $addProduct;
  private $addRegion;

  public function __construct(
    Environment $environment,
    Util $util,
    Service $service,
    AddProduct $addProduct,
    AddRegion $addRegion
  ) {
    $this->environment  = $environment;
    $this->util         = $util;
    $this->service      = $service;
    $this->addProduct   = $addProduct;
    $this->addRegion    = $addRegion;
  }

  public function updateProduct(Request $request, Response $response) {
    try {
      $data = $request->getParameters();

      foreach ($data as $product) {
        $this->addProduct->validateProduct($product, $response);

        $product["CDOPERADOR"] = $this->environment->getCdoperador();
        $product["IDATUREG"]  = empty($product["IDATUREG"])  ? "T" : "S";
        $product["IDATUFORN"] = empty($product["IDATUFORN"]) ? "T" : "S";

        if ($product["IDUTINDICECOL"] === "S") {
          $product["VRINDICEITMCOL"] = (float) $product["VRINDICEITMCOL"];
        } else {
          $product["VRPRECUNIITM"] = (float) $product["VRPRECUNIITM"];
        }

        if (!empty($product["DSJUSTITM"])) {
          $conditionsArray = [
            "CDFILIAL"    => $product["CDFILIAL"],
            "NRCOLEPRECO" => $product["NRCOLEPRECO"],
            "CDREGIAO"    => $product["CDREGIAO"],
            "CDFORNECED"  => $product["CDFORNECED"],
            "CDPRODUTO"   => $product["CDPRODUTO"],
            "NRORG"       => $product["NRORG"]
          ];

          $unitPrice = $this->util->filteredDataSet($conditionsArray, "getUnitPrice")[0]["VRPRECUNIITM"];

          if ($_SESSION["_sf2_attributes"]["LOGIN_LANGUAGE"] === "pt-br") {
            $product["DSALTERITM"] = "Preço Unitário - de: " . $unitPrice . " para " . $product["VRPRECUNIITM"];
          } else if ($_SESSION["_sf2_attributes"]["LOGIN_LANGUAGE"] === "en-us") {
            $product["DSALTERITM"] = "Unit Price - from: " . $unitPrice . " to " . $product["VRPRECUNIITM"];
          } else {
            $product["DSALTERITM"] = "Precio unitario - de: " . $unitPrice . " a " . $product["VRPRECUNIITM"];
          }

          $this->service->insertJustification($product);
        }

        if (!empty($this->verifySupplierMeets($product))) return $response->setError(new Error("sup::SUPPLIERS_NOT_SERVING_REGIONS", 400));

        if ($product["IDUTINDICECOL"] === "S") {
          $product["VRPRECUNIITM"] = $this->addProduct->calculateVrIndice($product);
          $this->service->updRegSupIndex($product);
        } else {
          $this->service->updateRegSup($product);
        }

        $this->addRegion->executeProcedureItemcol($product);
        $this->addRegion->processInsertFormPgtoData($product);

        $this->addProduct->executeProcPrices($product);

        $this->service->updatePreEcZerado($product);

        $productSupplyTek = $this->getSingleProdOnSupplyTek($product);

        if (sizeof($productSupplyTek) > 0) {
          $this->saveNotificationOnSupplyTek($product, $productSupplyTek, "single");
        }
      }

      $this->service->updateQuotationParams($data[0]);

      $brandChanged = $this->getChangedBrand($product);

      if (!empty($brandChanged)) {
        return $response->addDataSet(new DataSet("addProduct", ["message" => "Atualização do produto realizado com sucesso.", "status_code" => 200]));
      }

      $product["DSALTERITM"] = $product["DSJUSTITM"] ?? "";

      $this->service->updateChangedBrand($product);

      return $response->addDataSet(new DataSet("addProduct", ["message" => "Atualização do produto realizado com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      $response->setError(new Error($exception->getMessage(), 500));
    }
  }

  private function verifySupplierMeets(array $data): array {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "CDPRODUTO"   => $data["CDPRODUTO"],
      "CDMARCA"     => $data["CDMARCA"],
      "NRORG"       => $data["NRORG"]
    ];

    $region = $this->util->filteredDataSet($conditionsArray, "verifySupplierMeets");

    return $region;
  }

  private function getSingleProdOnSupplyTek(array $data): array {
    $conditionsArray = [
      "CDPRODUTO"   => $data["CDPRODUTO"],
      "CDMARCA"     => $data["CDMARCA"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "CDFILIAL"    => $data["CDFILIAL"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "NRORG"       => $data["NRORG"]
    ];

    $productSupplyTek = $this->util->filteredDataSet($conditionsArray, "verifySingleProdOnSupplyTek");

    return $productSupplyTek;
  }

  public function saveNotificationOnSupplyTek($data, $products, $type) {
    $data["NRNOTIFICSUPPLY"]   = $this->generateCode("NOTIFICSUPPLY");
    $data["CDFILIALSUPPLYTEK"] = $products[0]["CDFILIAL"];
    $data["CDOPERADOR"]        = $products[0]["CDOPERADOR"];
    $data["VRREAJUSTE"]        = isset($data["REAJUSTE"]) ? $data["REAJUSTE"] : null;

    if ($type == "single") {
      $data["NMNOTIFICACAO"] = "ASSUNTO_ALTERA_PRECO_UNICO";
      $data["DSJUSTIFIC"] = $data['DSJUSTITM_inline'];
      $justify = "";

      if ($data["DSJUSTITM_inline"] != "") {
        $justify = "com a seguinte justificativa: <b>\"" . $data["DSJUSTITM_inline"] ."\"</b> ";
      }

      $data["DSTEXTNOTIFIC"] = "TEXTO_ALTERA_PRECO_UNICO";
      $data["VRPRECOUNIOLD"] = !isset($data['VRINDICEITMCOL']) ? $data['changed_VRPRECUNIITM'] : $data['changed_VRINDICEITMCOL'];
      $data["VRPRECOUNINEW"] = !isset($data['VRINDICEITMCOL']) ? $data['VRPRECUNIITM_inline'] : $data['VRINDICEITMCOL_inline'];
    } else {
      $data["NMNOTIFICACAO"] = "ASSUNTO_ALTERA_PRECO_MULTIPLOS";
      $data["DSJUSTIFIC"] = null;
      $data["DSTEXTNOTIFIC"] = "TEXTO_ALTERA_PRECO_MULTIPLOS";
      $data["VRPRECOUNIOLD"] = null;
      $data["VRPRECOUNINEW"] = null;
      $data["NMMARCA"] = null;
      $data["NMPRODUTO"] = null;
    }

    $this->service->insertNotificationSupplyTek($data);
  }

  private function getChangedBrand(array $data): array {
    $conditionsArray = [
      "CDMARCA"     => $data["CDMARCA"],
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "CDREGIAO"    => $data["CDREGIAO"],
      "CDPRODUTO"   => $data["CDPRODUTO"],
      "NRORG"       => $data["NRORG"]
    ];

    $brand = $this->util->filteredDataSet($conditionsArray, "verifySingleProdOnSupplyTek");

    return $brand;
  }

  private function searchBrand(array $data): array {
    $conditionsArray = [
      "CDARVPROD" => $data["CDARVPROD"],
      "CDMARCA"   => $data["CDMARCA"],
      "CDMARCA"   => $data["CDMARCA"],
      "NRORG"     => $data["NRORG"]
    ];

    $brand = $this->util->filteredDataSet($conditionsArray, "distinctBranchProduct");

    return $brand;
  }
}
