import {
  Modal, ModalService, SelectMultiple, DialogService, LoadingService, Button,
} from '@zeedhi/common';
import {
  Http, Metadata, IDictionary, FormatterParserProvider,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

FormatterParserProvider.registerFormatter('column_routineMask', ({ row }: { row: IDictionary }) => {
  let mask = '';
  if (row.ROUTINE === 'ECOMMERCEAF') {
    mask = 'E-commerce AF';
  }

  return mask;
});

export class ReportConfigController {
  private reportModal!: Modal;

  private utilities!: Utilities;

  private reportSelect!: SelectMultiple;

  public async createCrudModal() {
    this.reportModal = await ModalService.createFromJson(
      '/modals/reportConfig/addReportModal',
      true,
    );
  }

  private grid() {
    return Metadata.getInstance<TekGrid>('reportConfig');
  }

  public destroyCrudModal() {
    this.reportModal?.destroy();
  }

  public async openCrudModal() {
    await this.reportModal.show();
    this.reportSelect = Metadata.getInstance<SelectMultiple>('reportSelect');
  }

  public closeModal() {
    this.reportModal.hide();
    this.reportSelect.unSelectAllItems();
  }

  public async saveReport() {
    LoadingService.show();
    Http.post('/addReport', this.reportSelect.value)
      .then((value) => {
        if (value.data.error) {
          this.showErrorDialog(value.data.error);
        } else {
          this.showSuccessDialog(value.data.dataset.addReport.message);
          this.grid().reload();
        }
      })
      .catch((error) => {
        this.showErrorDialog(error);
      }).finally(() => {
        LoadingService.hide();
        this.closeModal();
      });
  }

  private showSuccessDialog(message) {
    DialogService.show({
      name: 'success-dialog',
      type: 'success',
      title: 'Sucesso',
      text: message,
    });
  }

  private showErrorDialog(error) {
    DialogService.show({
      name: 'error-dialog',
      type: 'error',
      title: 'Erro',
      text: error,
    });
  }

  public async deleteReport() {
    try {
      LoadingService.show();
      const { data } = await Http.post('/deleteReport', this.grid().selectedRows);
      LoadingService.hide();
      this.showSuccessDialog(data.dataset.deleteReport.message);
      this.grid().reload();
    } catch (error) {
      this.showErrorDialog(error);
    }
  }

  public enableButtons() {
    const saveButton = Metadata.getInstance<Button>('saveButton');
    if (this.reportSelect.isValid()) {
      saveButton.disabled = false;
    } else {
      saveButton.disabled = true;
    }
  }
}
