import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  Date as DateInput, Form, ModalService,
} from '@zeedhi/common';
import { Metadata, dayjs } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { IFilterData, ISupplierFilter } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class FilterController {
  private appState: AppState;

  constructor() {
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/filterModal',
      true,
    );
    this.openModal();
  }

  public destroyModal(): void {
    ModalService.getModal('filterModal')?.destroy();
  }

  public openModal(): void {
    ModalService.getModal('filterModal').show();
  }

  public closeModal(): void {
    ModalService.getModal('filterModal').hide();
  }

  public getValidityDate(): void {
    const date = Metadata.getInstance<DateInput>('filterValidity');
    date.value = [dayjs(), dayjs()];
  }

  public get filterSupplier(): ISupplierFilter {
    return {
      P_NRORG: this.appState.NRORG,
      P_IDULTIDTVALQ: 'T',
      P_DTCOL: 'T',
    };
  }

  public async filterDatasource(): Promise<void> {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const data = this.processData();

    quoteGrid.datasource.setFilter({
      P_CDFILIAL: this.appState.CDFILIAL,
      P_TCOMPRADOR: Array.isArray(data.buyer)
        ? null
        : data.buyer,
      P_TREGIAO: Array.isArray(data.region)
        ? null
        : data.region,
      P_TFORN: Array.isArray(data.supplier)
        ? null
        : data.supplier,
      P_TDATAS: Array.isArray(data.validity)
        ? null
        : data.validity,
      P_CDCOMPRADOR: Array.isArray(data.validity)
        ? data.buyer
        : null,
      P_CDREGIAO: Array.isArray(data.region)
        ? data.region
        : null,
      P_CDFORNECED: Array.isArray(data.validity)
        ? data.supplier
        : null,
      P_DTINIVALCOL: Array.isArray(data.validity)
        ? data.validity[0]
        : null,
      P_DTFINVALCOL: Array.isArray(data.validity)
        ? data.validity[1]
        : null,
      P_NRORG: this.appState.NRORG,
    });

    ModalService.closeByName('filterModal');
  }

  private processData(): IFilterData {
    const form = Metadata.getInstance<Form>('filterForm');

    return {
      buyer: form.value.filterBuyer.length ? form.value.filterBuyer : 'T',
      region: form.value.filterRegion.length ? form.value.filterRegion : 'T',
      supplier: form.value.filterSupplier.length ? form.value.filterSupplier : 'T',
      validity: form.value.filterValidity.length ? form.value.filterValidity : 'T',
    };
  }
}
