import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService, LoadingService, Form,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class LoadBestPricesController {
  private utilities!: Utilities;

  constructor() {
    this.utilities = new Utilities();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/loadBestPricesModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('loadBestPricesModal')?.destroy();
  }

  public openModal(): boolean {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!quoteGrid.selectedRows.length) {
      this.utilities.showDialog(
        'Selecione uma cotação para realização do carregamento dos melhores preços.',
        'warning',
      );

      return false;
    }

    ModalService.getModal('loadBestPricesModal').show();

    return true;
  }

  public closeModal(): void {
    ModalService.getModal('loadBestPricesModal').hide();
  }

  private async loadBestPrices(): Promise<void> {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const productGrid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
    const form = Metadata.getInstance<Form>('loadBestPricesForm');

    if (!form.validate()) return;

    const params = quoteGrid.selectedRows.map((row) => ({
      ...row,
      FORM_DTINIVALCOL: form.value.loadBestPricesValidity[0],
      FORM_DTFINVALCOL: form.value.loadBestPricesValidity[1],
    }));

    LoadingService.show();

    try {
      await Http.post('/loadBestPrices', params);

      this.closeModal();
      this.utilities.showDialog('sup::SUCCESS', 'success');

      productGrid.reload();
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    } finally {
      LoadingService.hide();
    }
  }
}
