import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService, Button, Textarea, LoadingService,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class ObservationController {
  private utilities!: Utilities;

  constructor() {
    this.utilities = new Utilities();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/observationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('observationModal')?.destroy();
  }

  public openModal(): void {
    const productGrid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');

    if (!productGrid.selectedRows.length || productGrid.selectedRows.length > 1) {
      this.utilities.showDialog(
        'Selecione um produto para visualização da observação.',
        'warning',
      );

      return;
    }

    ModalService.getModal('observationModal').show();

    this.setObservation(productGrid.selectedRows[0]);
  }

  public closeModal(): void {
    ModalService.getModal('observationModal').hide();
  }

  private async setObservation(row): Promise<void> {
    LoadingService.show();

    try {
      const { data } = await Http.get('/getObservation', { params: row });
      const observation = Metadata.getInstance<Textarea>('observation');

      observation.value = data.dataset.observation.DSOBSITMCOL;
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', error);
    } finally {
      LoadingService.hide();
    }
  }

  public enableSaveButton(): void {
    const saveButton = Metadata.getInstance<Button>('saveObservation');

    saveButton.disabled = false;
  }

  public async saveObservation(): Promise<void> {
    try {
      const product = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
      const observation = Metadata.getInstance<Textarea>('observation');
      const row = product.datasource.currentRow;
      const parameters = { ...row, DSOBSITMCOL: observation.value };

      await Http.put('/saveObservation', { params: parameters });

      this.utilities.showDialog('sup::SUCCESS', 'success');
      this.closeModal();
    } catch (error) {
      this.utilities.showDialog('sup::ERROR', 'error');
    }
  }
}
