import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Metadata } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import type {
  IFilterQuotationProductsMasterDetail,
  IFilterSupplierProductsMasterDetail,
} from '@/controllers/interfaces/QuoteSheetGrid';

export class GridFiltersController {
  private appState: AppState;

  constructor() {
    this.appState = AppState.getInstance();
  }

  public get filterBuyer(): { P_CDOPERADOR: string } {
    return { P_CDOPERADOR: this.appState.CDOPERADOR };
  }

  public filterQuoteMasterDetail(): void {
    const quotationGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const quoteSheetGrid = Metadata.getInstance<TekGrid>('quoteSheetGrid');

    quotationGrid.datasource.setFilter(quoteSheetGrid.datasource.filter);
  }

  public get filterQuotationProductsMasterDetail(): IFilterQuotationProductsMasterDetail {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    return {
      P_NRCOLEPRECO: quoteGrid.datasource.currentRow?.NRCOLEPRECO ?? null,
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRORG: this.appState.NRORG,
    };
  }

  public get filterSupplierProductsMasterDetail(): IFilterSupplierProductsMasterDetail {
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');

    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRCOLEPRECO: supplierGrid.datasource.currentRow?.NRCOLEPRECO ?? null,
      P_CDREGIAO: regionGrid.datasource.currentRow?.CDREGIAO ?? null,
      P_CDFORNECED: supplierGrid.datasource.currentRow?.CDFORNECED ?? null,
      P_NRORG: this.appState.NRORG,
    };
  }
}
